@@ -37,7 +37,6 @@ impl<'a> Emitter<'a> {
         space!();
         emit!(node.ident);
         emit!(node.class.type_params);
-        formatting_space!();
 
         self.emit_class_trailing(&node.class)?;
     }
@@ -117,6 +116,15 @@ mod tests {
         );
     }
 
+    #[test]
+    fn issue_1764() {
+        assert_min(
+            "class Hoge {};
+class HogeFuga extends Hoge {};",
+            "class Hoge{};class HogeFuga extends Hoge{};",
+        );
+    }
+
     #[test]
     fn single_argument_arrow_expression() {
         assert_min("function* f(){ yield x => x}", "function*f(){yield x=>x}");
