@@ -7,7 +7,7 @@ use self::{
     text_writer::WriteJs,
     util::{SourceMapperExt, SpanExt, StartsWithAlphaNum},
 };
-use std::{fmt::Write, io, sync::Arc};
+use std::{borrow::Cow, fmt::Write, io, sync::Arc};
 use swc_atoms::JsWord;
 use swc_common::{
     comments::Comments, sync::Lrc, BytePos, SourceMap, Span, Spanned, SyntaxContext, DUMMY_SP,
@@ -1461,7 +1461,8 @@ impl<'a> Emitter<'a> {
         self.emit_leading_comments_of_span(ident.span, false)?;
 
         // TODO: span
-        self.wr.write_symbol(ident.span, &ident.sym)?;
+        self.wr
+            .write_symbol(ident.span, &handle_invalid_unicodes(&ident.sym))?;
         if ident.optional {
             punct!("?");
         }
@@ -2468,8 +2469,9 @@ fn unescape_tpl_lit(s: &str) -> String {
 
 fn escape_without_source(v: &str, target: JscTarget, single_quote: bool) -> String {
     let mut buf = String::with_capacity(v.len());
+    let mut iter = v.chars().peekable();
 
-    for c in v.chars() {
+    while let Some(c) = iter.next() {
         match c {
             '\u{0008}' => buf.push_str("\\b"),
             '\u{000c}' => buf.push_str("\\f"),
@@ -2479,7 +2481,14 @@ fn escape_without_source(v: &str, target: JscTarget, single_quote: bool) -> Stri
             '\u{000b}' => buf.push_str("\\v"),
             '\0' => buf.push_str("\\x00"),
 
-            '\\' => buf.push_str("\\\\"),
+            '\\' => {
+                if iter.peek() == Some(&'\0') {
+                    buf.push_str("\\");
+                    iter.next();
+                } else {
+                    buf.push_str("\\\\")
+                }
+            }
 
             '\'' if single_quote => buf.push_str("\\'"),
             '"' if !single_quote => buf.push_str("\\\""),
@@ -2500,20 +2509,7 @@ fn escape_without_source(v: &str, target: JscTarget, single_quote: bool) -> Stri
             }
 
             _ => {
-                if target >= EsVersion::Es2015 {
-                    let escaped = c.escape_unicode().to_string();
-
-                    if escaped.starts_with('\\') {
-                        buf.push_str("\\u");
-                        if escaped.len() == 8 {
-                            buf.push_str(&escaped[3..=6]);
-                        } else {
-                            buf.push_str(&escaped[2..]);
-                        }
-                    }
-                } else {
-                    buf.push(c);
-                }
+                buf.push(c);
             }
         }
     }
@@ -2582,6 +2578,12 @@ fn escape_with_source<'s>(
         }
 
         if orig_c == '\\' {
+            if s_iter.as_str().starts_with("\\\0") {
+                for _ in 0..6 {
+                    s_iter.next();
+                }
+            }
+
             buf.push('\\');
             match orig_iter.next() {
                 Some('\\') => {
@@ -2697,6 +2699,14 @@ fn is_single_quote(cm: &SourceMap, span: Span) -> Option<bool> {
     Some(single_quote)
 }
 
+fn handle_invalid_unicodes(s: &str) -> Cow<str> {
+    if !s.contains("\\\0") {
+        return Cow::Borrowed(s);
+    }
+
+    Cow::Owned(s.replace("\\\0", "\\"))
+}
+
 #[cold]
 #[inline(never)]
 fn invalid_pat() -> ! {
